# -*- coding: utf-8 -*-
#Copyright Jon Berg , turtlemeat.com

import string,cgi,time,sys
import os
import datetime
import threading
from socket import gethostname
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
import that_module
import main

SERVER_PORT = 8080
SERVER_HOST = ''

PATH_WWW      = "www/"
PATH_BASE     = "builtin/"
PATH_MODULES  = "modules/"
HTML_WRAPPER  = "wrapper.html"
HTML_HEADER   = "header.html"
HTML_MENUMAIN = "menu_main.html"

def print_info(text):
    print "[ INFO ] " + text
def print_error(text):
    print "[ ERROR] " + text
def print_warning(text):
    print "[ WARN ] " + text

class page_generator:

    def __init__(self):
        self.html_output = []       # A new list of strings for the output
        self.load_wrapper()

    def load_wrapper(self):
        content = self.load_file(PATH_BASE + HTML_WRAPPER)
        if (content == -1): return

        head, foot = self.split_html(content, "[[Content]]")

        if (head == "" or foot == ""):
            head = "<html>"
            foot = "</html>"
            print_warning("HTML template file was parsed incorrectly")

        # Append the html header and footer code into the
        # list of strings to be output later... 
        self.html_output.append(head)
        self.html_output.append(foot)

    def load_template(self, header, menu):
        if (header == 1):
            content = self.load_file(PATH_BASE + HTML_HEADER)
            if (content == -1): return
            self.html_output.append(content)

        if (menu == 1):
            content = self.load_file(PATH_BASE + HTML_MENUMAIN)
            if (content == -1): return
            self.html_output.append(content)

    def load_copta(self):
        self.load_template(1,1)
        content = self.load_file("modules/copta-module.html")
        if (content == -1):
            self.html_output.append("ERROR reading index.html")
            return (-1)

        obj = that_module.mod_objs[0]

        content = self.replace_tag(content, "[[ALIAS]]", obj.alias)
        content = self.replace_tag(content, "[[FULLNAME]]", obj.mod_fullname)
        content = self.replace_tag(content, "[[ALIAS]]", obj.alias)

        content = self.parse_control_tags(obj, content)
        self.html_output.append(content)

    def parse_control_tags(self, modobj, content):
        # Search for and handle all [[x]] tags in html file
        while(1):
            first, delim, last = content.partition("[[")
            if (delim == "" or last == ""): break
        
            tag, delim, last  = last.partition("]]")
            if (delim == "" or last == ""): break
           
            tagbr = "[[" + tag + "]]"
            content = self.replace_tag(content, tagbr, modobj.get_value_by_name(tag))
        return content

    def load_index(self):
        self.load_template(1,1)
        content = self.load_file("index.html")
        if (content == -1):
            self.html_output.append("ERROR reading index.html")
            return (-1)

        content = self.replace_tag(content, "[[TIME]]", main.get_time())
        content = self.replace_tag(content, "[[DATE]]", main.get_date())
        self.html_output.append(content)

    def load_status(self):
        self.load_template(1,1)
        content = self.load_file("status.html")
        if (content == -1):
            self.html_output.append("ERROR reading status.html")
            return (-1)
        content = self.replace_tag(content, "[[TIME]]",     main.get_time())
        content = self.replace_tag(content, "[[DATE]]",     main.get_date())
        content = self.replace_tag(content, "[[HOSTNAME]]", gethostname())
        content = self.replace_tag(content, "[[OSNAME]]",   main.OSNAME)
        content = self.replace_tag(content, "[[VERSION]]",  main.VERSION)
        content = self.replace_tag(content, "[[BUILDDATE]]",main.BUILD)
        self.html_output.append(content)


    def load_file(self, filename):
        try:
            print_info("Opening HTML file: " + PATH_WWW + filename)
            fileobject = open(PATH_WWW + filename)
            htmlinput = fileobject.read()
            fileobject.close()
            print_info("HTML file read successfully")
            return (htmlinput)
        except:
            print_error("Unable to load HTML file")
            return (-1)

    def load_content(self, content):
        self.html_output.append(content)

    def split_html(self, content, delimiter):
        beginning = ""
        delim = ""
        end = ""

        # Split the string up into 3 parts
        beginning, delim, end = content.partition(delimiter)
        return beginning, end

    def replace_tag(self, content, delimiter, replacement):
        temp_content = content
        beginning = ""
        delim = ""
        end = ""

        while(True):
            # Split the string up into 3 parts
            beginning, delim, end = temp_content.partition(delimiter)

            # Delimiter tag was not found, return unaltered content
            if (delim == "" or end == ""): return (temp_content)
            temp_content = beginning + str(replacement) + end

        return (temp_content)

    def send(self):
        output = ""
        output = self.html_output[0]              # Send beginnning
        for i in range(2, len(self.html_output)):
            output = output + self.html_output[i] # Send content
        output = output + (self.html_output[1])   # Send ending
        return output


class http_handler (BaseHTTPRequestHandler):
    # HTTP GET request handler
    def do_GET(self):
        print_info("HTTP GET request for " + self.path)

        try:
            if (self.path == "/") or (self.path == "/index.html"):
                newpage = page_generator()
                result  = newpage.load_index()

                if (result == -1): raise IOError;

                self.send_response(200)
                self.send_header('Content-type','text/html')
                self.end_headers()
                self.wfile.write(newpage.send())

            elif self.path.endswith(".css"):
                path = self.path[1:len(self.path)]
                try:
                    print_info("Opening CSS file: " + PATH_WWW + path)
                    fileobject = open(PATH_WWW + path)
                    cssinput = fileobject.read()
                    fileobject.close()
                except:
                    print_error("Reading of CSS file failed.")
                    return

                self.send_response(200)
                self.send_header('Content-type','text/css')
                self.end_headers()
                self.wfile.write(cssinput)

            elif self.path.endswith(".esp"):   #our dynamic content
                self.send_response(200)
                self.send_header('Content-type','text/html')
                self.end_headers()
                self.wfile.write("hey, today is the" + str(time.localtime()[7]))
                self.wfile.write(" day in the year " + str(time.localtime()[0]))
                return
                
            else:
                newpage = page_generator()

                if (self.path == "/status.html"):
                    result = newpage.load_status()
                elif (self.path == "/modules/copta-module.html"):
                    result = newpage.load_copta()
                elif (self.path == "/modules/main-thermostat"):
                    result = newpage.load_copta()
                else:
                    result = newpage.load_file(self.path[1:len(self.path)])

                if (result == -1): raise IOError;

                try:
                    if (self.send_headers == 1):
                        pass
                except:
                    self.send_headers = 0

                if (self.send_headers == 1):
                        self.send_headers = 0
                else:
                    self.send_response(200)
                    self.send_header('Content-type','text/html')
                    self.end_headers()
                self.wfile.write(newpage.send())

            return
                
        except IOError:
            self.send_error(404,'File Not Found: %s' % self.path)
     
    # HTTP POST request handler
    def do_POST(self):
        global rootnode

        print_info("HTTP POST received")

        try:
            ctype, pdict = cgi.parse_header(self.headers.getheader('content-type'))
            if ctype == 'multipart/form-data':
                query=cgi.parse_multipart(self.rfile, pdict)
            self.send_response(301)
            self.end_headers()

            # Get alias value from the dictionary, which returns the value
            # as a list element, so then get the first list element
            modulealias = query['alias']
            modulealias = modulealias[0]
            
            # Remove the alias dictionary element
            del query['alias']

            # Convert the remaining dictionary elements into an array
            # of strings representing the key and value pairs like:
            # query[0] = ('key',['value'])
            query    = dict.items(query)
            length   = len(query)
            
            # Handle each element one-by-one...
            for i in range(0, length):

                controlname = str(query[i])
                #sep1, sep2, controlname = controlname.partition(",")

                # Get just the control name now
                controlname, sep, controldata = controlname.partition(",")
                sep1  = controlname.find ("'")
                sep2  = controlname.rfind("'")
                controlname = controlname[sep1+1 : sep2]

                # Get just the control value now
                sep1 = controldata.find("['")
                sep2 = controldata.find("']")
                controldata = controldata[(sep1+2): sep2]

                print_info("Control: \"" + controlname + "\" to be set to: " + controldata)

                modobj = that_module.mod_objs[0]
                modobj.set_value_by_name(controlname, controldata)

            self.path = "/modules/copta-module.html"
            self.send_headers = 1
            self.do_GET()

        except :
            print_error("Error in POST data")

    def send_error(self, number, text):
        self.send_response(number)
        self.send_header('Content-type','text/html')
        self.end_headers()

        if (number == 404): self.wfile.write("<h2>404 Error</h2>")
        self.wfile.write(text)
