# -*- coding: utf-8 -*-
from xml.dom.minidom import parse, parseString
import sys, string

class xml_parser:

    def __init__(self, xml_file, verbose):
        # 0: no logging
        # 1: errors only
        # 2: errors + warnings only
        # 3: errors + warnings + info
        self.verbose = verbose

        self.mod_shortname    = ""
        self.mod_fullname     = ""
        self.mod_class        = ""
        self.mod_description  = ""
        self.mod_version      = ""
        self.mod_id           = ""

        self.manu_name     = ""
        self.manu_location = ""
        self.manu_date     = ""
        self.manu_phone    = ""
        self.manu_fax      = ""
        self.manu_website  = ""
        self.manu_email    = ""

        # A group of lists will hold the information corresponding to
        # every control available for read/write on a THAT module, and
        # the possible states, min/max value, current value, etc.
        self.control  = []  # Textual Name of a control
        self.register = []  # Integer or low-level name of a control
        self.states   = []  # Possible states (or "--")
        self.type     = []  # Read/write setting
        self.min      = []  # Minimum value for this control (or "--")
        self.max      = []  # Maximum value for this control (or "--")
        self.defaultv = []  # Default value
        self.defaults = []  # Default State
        self.current  = []  # The current value of this control

        try:
            xml_dom = parse(xml_file)
            self.print_info("XML file " + xml_file + " loaded")
            self.parse_module(xml_dom)
        except IOError:
            self.print_error("XML file \"" + xml_file + "\" could not be loaded!")
        except Exception, error:
            self.print_error("XML error in file: \"" + xml_file + "\" : " + str(error))

    def print_info(self, text):
        if (self.verbose >= 3):
            print "[ INFO ] " + text
    def print_error(self, text):
        if (self.verbose >= 1):
            print "[ ERROR] " + text
    def print_warning(self, text):
        if (self.verbose >= 2):
            print "[ WARN ] " + text

    def getText(self, nodelist):
        rc = ""
        for node in nodelist:
            if node.nodeType == node.TEXT_NODE:
                rc = rc + node.data
        return rc

    def parse_module(self, module):
        # Check XML file for a <module> element
        mod = self.get_element(module, "module")
        if (mod == "---"): return

        # Check XML file for a name="" attribute
        self.mod_shortname = self.get_attribute(mod, "name")
        if (self.mod_shortname == "---"): return

        # Handle the <identification> element, if it exists        
        stat = self.parse_identification(module)
        if (stat == "---"): return

        # Handle the <manufacturer> element, if it exists
        self.parse_manufacturer(module)

        # Handle the <controls> element, if it exists
        stat = self.parse_controls(module)
        if (stat == "---"): return

    def parse_identification(self, module):
        # Check XML file for a <identification> element
        ident = self.get_element(module, "identification")
        if (ident == "---"): return("---")

        self.mod_id = self.get_attribute(ident, "id")
        if (self.mod_id == "---"): return("---")

        self.mod_fullname    = self.get_element_data(ident, "fullname")
        self.mod_description = self.get_element_data(ident, "description")
        self.mod_version     = self.get_element_data(ident, "version")
        self.mod_class       = self.get_element_data(ident, "class")

    def parse_manufacturer(self, module):
        # Check XML file for a <manufacturer> element, if it exists
        manu = self.get_element(module, "manufacturer")
        if (manu == "---"): return

        self.manu_name     = self.get_element_data(manu, "name")
        self.manu_location = self.get_element_data(manu, "location")
        self.manu_date     = self.get_element_data(manu, "date")
        self.manu_phone    = self.get_element_data(manu, "phone")
        self.manu_fax      = self.get_element_data(manu, "fax")
        self.manu_website  = self.get_element_data(manu, "website")
        self.manu_email    = self.get_element_data(manu, "email")

    def parse_controls(self, module):
        # Read <controls> section of XML file
        # Read <controls-item> sections from <controls> section
        controls     = self.get_element(module, "controls")

        if (controls == "---"): return
        controlitems = controls.getElementsByTagName("controls-item")
        # Parse each <controls-item> separately
        for item in controlitems:
            self.parse_control(item)

    def parse_control(self, control):
        print("")
        name     = control.getAttribute("name")
        self.control.append(name)
        self.print_info("Found new control  : " + name)

        register = self.get_element_data(control, "register")
        self.register.append(register)
        self.print_info("---> Register      : " + register)

        type     = self.get_element_data(control, "type")
        self.type.append(type)
        self.print_info("---> Type          : " + type)

        values   = self.get_element(control, "values")
        if (values != "---"):
            default = values.getAttribute("default")
            min      = self.get_element_data(control, "min")
            max      = self.get_element_data(control, "max")
            self.current.append(default)
        else:
            default = "---"
            min     = "---"
            max     = "---"

        self.defaultv.append(default)
        self.min.append(min)
        self.max.append(max)

        self.print_info("---> Default Value : " + default)
        self.print_info("---> Minimum       : " + min)
        self.print_info("---> Maximum       : " + max)

        statelist = []
        states   = self.get_element(control, "states")
        if (states != "---"):
            try:
                default = states.getAttribute("default")
                if (default == ""): raise Exception
            except:
                self.print_error("XML <control> element has no/invalid default attribute. Parsing Stopped.")
                return

            statesitems = states.getElementsByTagName("states-item")
            for state in statesitems:
                try:
                    state_name  = state.getAttribute("name")
                    if (state_name == ""): raise Exception
                except:
                    self.print_warning("XML <states-item> element has no/invalid name.")

                state_value = self.get_element_data(state, "value")

                #self.print_info("---> State        : " + state_name)
                #self.print_info("------> Value     : " + state_value)

                statelist.append([state_name, state_value])
                # If the current state's name matches the default...
                if (state_name == default):
                    self.defaults.append(state_value)
                    self.current.append(state_value)

            self.print_info("---> Default state : " + default)
            self.states.append(statelist)

        # This control is NOT state-based, so return --- for all state values
        else:
            statelist = "---"
            self.states.append("---")
            self.defaults.append("---")

        self.print_info("---> States        : " + str(statelist))

        print("")

    def get_attribute(self, element, attribute):
        try:
            data = element.getAttribute(attribute)
            if (data == ""): raise Exception
        except:
            self.print_error("XML <" + str(element.nodeName) + \
            "> element has invalid \"" + attribute + \
            "\" attribute. Parsing Stopped.")
            return("---")
        return(data)

    def get_element_data(self, parent, tagname):
        try:
            object = parent.getElementsByTagName(tagname)[0]
            data   = self.getText(object.childNodes)
            return (data)
        except:
            #self.print_warning("Couldn't find tag: " + tagname)
            return ("---")

    def get_element(self, parent, tagname):
        try:
            object = parent.getElementsByTagName(tagname)[0]
            return (object)
        except:
            self.print_error("XML <" + tagname + \
            "> element Not Found.")
            return ("---")
