# -*- coding: utf-8 -*-

from xml.dom.minidom import parse, parseString
import sys
import string

def print_info(text):
    print "[ INFO  ] " + text
def print_error(text):
    print "[ ERROR ] " + text
def print_warning(text):
    print "[ WARN  ] " + text

def main(args):

    dom = parse("modules/rly1-module.xml")
    parse_module(dom)


def getText(nodelist):
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc


def parse_module(module):

    # Read <controls> section of XML file
    # Read <controls-item> sections from <controls> section
    controls     = module.getElementsByTagName("controls")[0]
    control      = controls.getElementsByTagName("controls-item")
    parse_controls(control)

def parse_controls(controls):
    # Parse each <controls-item> separately
    for control in controls:
        parse_control(control)


def parse_control(control):
    print("")
    name     = control.getAttribute("name")
    print_info("Found new control  : " + name)
    
    register = get_element_data(control, "register")
    print_info("---> Register      : " + register)

    type     = get_element_data(control, "type")
    print_info("---> Type          : " + type)
    
    
    values   = get_element(control, "values")
    if (values != "---"):
        default = values.getAttribute("default")
        min      = get_element_data(control, "min")
        max      = get_element_data(control, "max")
    else:
        default = "---"
        min     = "---"
        max     = "---"
    print_info("---> Default Value : " + default)
    print_info("---> Minimum       : " + min)
    print_info("---> Maximum       : " + max)

    statelist = []
    states   = get_element(control, "states")
    if (states != "---"):
        default = states.getAttribute("default")
        statesitems = states.getElementsByTagName("states-item")
        for state in statesitems:
            state_name  = state.getAttribute("name")
            state_value = get_element_data(state, "value")

            #print_info("---> State        : " + state_name)
            #print_info("------> Value     : " + state_value)

            statelist.append([state_name, state_value])

        print_info("---> Default state : " + default)

    else:
        statelist = "---"

    print_info("---> States        : " + str(statelist))

    print("")


def get_element_data(control, tagname):
    try:
        object = control.getElementsByTagName(tagname)[0]
        data   = getText(object.childNodes)
        return (data)
    except:
        #print_warning("Couldn't find tag: " + tagname)
        return ("---")

def get_element(control, tagname):
    try:
        object = control.getElementsByTagName(tagname)[0]
        return (object)
    except:
        #print_warning("Couldn't find tag: " + tagname)
        return ("---")


def handleSlides(slides):
    for slide in slides:
        handleSlide(slide)

def handleSlide(slide):
    handleSlideTitle(slide.getElementsByTagName("title")[0])
    handlePoints(slide.getElementsByTagName("point"))

def handleSlideshowTitle(title):
    print "<title>%s</title>" % getText(title.childNodes)

def handleSlideTitle(title):
    print "<h2>%s</h2>" % getText(title.childNodes)

def handlePoints(points):
    print "<ul>"
    for point in points:
        handlePoint(point)
    print "</ul>"

def handlePoint(point):
    print "<li>%s</li>" % getText(point.childNodes)

def handleToc(slides):
    for slide in slides:
        title = slide.getElementsByTagName("title")[0]
        print "<p>%s</p>" % getText(title.childNodes)



if __name__ == '__main__':
    # Get any command-line arguments passed from the terminal
    arguments = []
    for arg in sys.argv:
        arguments.append(arg)

    main(arguments)