# -*- coding: utf-8 -*-

# ==========================================================================
# THAT Home Automation Topology -- THAT Control Software
# This file is part of THAT Control Software version 0.6.4 -- 2010.05.05
# Copyright (C) Nick Viera  ( http://www.nickviera.com/ )
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/> 
# ==========================================================================

import string,time,sys
import os
import datetime
import threading
import that_module
from web_server  import *

VERBOSE    = 1
VERSION    = "0.6.3"
BUILD      = "2010/04/02"
OSNAME     = os.name
server     = ""

def print_info(text):
    print "[ INFO ] " + text
def print_error(text):
    print "[ ERROR] " + text
def print_warning(text):
    print "[ WARN ] " + text

def get_time():
    timenow = time.strftime("%H:%M:%S", time.localtime())
    return(timenow)

def get_date():
    datenow = time.strftime("%Y/%m/%d", time.localtime())
    return(datenow)

def modules_load(path):
    dir_list = os.listdir(path)
    for file_name in dir_list:
        print file_name

def main(arguments):
    global SERVER_HOST
    global SERVER_PORT
    global VERBOSE
    global server

    def print_splash():
        print "\r\nTHAT Home Automation Control Software"
        print "Version 0.2 - 2010/03/09"
        print "Copyright (c) 2010 Nick Viera"
        print "http://www.tehhouse.us/electronics/that/"

    def print_menu():
        print_splash()
        print "\r\nValid command-line options are:"
        print "-h,  --help     Print this menu and then exit"
        print "     --port=N   Start the web server on port N"
        print "     --verbose  Run with verbose output\r\n"

    # Handle each command-line argument
    for i in range(0, len(arguments)):
        # Handle a blank argument
        if (arguments[i] == "" or arguments[i] == " "):
            pass

        # Handle the port number argument
        elif (arguments[i].startswith("--port=")):
            part1, part2, part3 = arguments[i].partition("=")
            SERVER_PORT = int(part3)
        
        # Handle help or invalid argument
        elif (arguments[i] == "--help" or arguments[i] == "-h"):
            print_menu()
            sys.exit(0)
 
    copta = that_module.that_mod("Main Thermostat")
    that_module.mod_names.append("copta")
    that_module.mod_objs.append(copta)

    print_info("Starting HTTP server on port " + str(SERVER_PORT))

    try:
        server = HTTPServer((SERVER_HOST, SERVER_PORT), http_handler)
        print_info("HTTP server started")
        server.serve_forever()

    except KeyboardInterrupt:
        sys.stdout.write("\b\b")
        print_info("Received keyboard interrupt")
        print_info("Stopping HTTP server")
        server.socket.close()
        print_info("Exiting program")
        sys.exit(0)

    except:
        print_error("The HTTP server died unexpectedly")


if __name__ == '__main__':
    # Get any command-line arguments passed from the terminal
    arguments = []
    for arg in sys.argv:
        arguments.append(arg)
    main(arguments)

