# -*- coding: utf-8 -*-

# ==========================================================================
# THAT Home Automation Topology -- THAT Control Software
# This file is part of THAT Control Software version 0.6.4 -- 2010.05.05
# Copyright (C) Nick Viera  ( http://www.nickviera.com/ )
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/> 
# ==========================================================================

import socket
from   xml_parser import *
from   main       import *

mod_names = []
mod_objs  = []

##########################################################################
class that_mod:

    def __init__(self, alias):
        self.socket   = ""
        self.alias    = alias

        self.mod_id           = ""
        self.mod_shortname    = ""
        self.mod_fullname     = ""
        self.mod_class        = ""
        self.mod_description  = ""
        self.mod_version      = ""
        self.addr             = 0
        self.port             = 0

        # A group of lists will hold the information corresponding to
        # every control available for read/write on a THAT module, and
        # the possible states, min/max value, current value, etc.
        self.control  = []  # Textual Name of a control
        self.register = []  # Integer or low-level name of a control
        self.states   = []  # Possible states (or "--")
        self.type     = []  # Read/write setting
        self.min      = []  # Minimum value for this control (or "--")
        self.max      = []  # Maximum value for this control (or "--")
        self.defaultv = []  # Default value
        self.defaults = []  # Default State
        self.current  = []  # The current value of this control

        print_info("Module " + self.alias + " loaded")
        xmlnow = xml_parser("modules/copta-module.xml", 3)
        self.load_xml_data(xmlnow)

    def load_xml_data(self, xml):
        self.mod_shortname    = xml.mod_shortname
        self.mod_fullname     = xml.mod_fullname
        self.mod_class        = xml.mod_class
        self.mod_description  = xml.mod_description
        self.mod_version      = xml.mod_version
        self.mod_id           = xml.mod_id

        self.manu_name        = xml.manu_name     
        self.manu_location    = xml.manu_location 
        self.manu_date        = xml.manu_date     
        self.manu_phone       = xml.manu_phone    
        self.manu_fax         = xml.manu_fax      
        self.manu_website     = xml.manu_website  
        self.manu_email       = xml.manu_email    

        self.control  = xml.control
        self.register = xml.register
        self.states   = xml.states
        self.type     = xml.type
        self.min      = xml.min
        self.max      = xml.max
        self.defaultv = xml.defaultv
        self.defaults = xml.defaults
        self.current  = xml.current

        print_info(self.mod_shortname)
        print_info(self.mod_fullname)
        print_info(self.mod_class)
        print_info(self.mod_description)
        print_info(self.mod_version)

        print_info(self.manu_name)
        print_info(self.manu_location)
        print_info(self.manu_date)
        print_info(self.manu_phone)
        print_info(self.manu_fax)
        print_info(self.manu_website)
        print_info(self.manu_email)

        print_info(str(self.control))
        print_info(str(self.register))
        print_info(str(self.states))
        print_info(str(self.type))
        print_info(str(self.min))
        print_info(str(self.max))
        print_info(str(self.defaultv))
        print_info(str(self.defaults))
        print_info(str(self.current))

    def get_value_by_name(self, name):
        index = ""

        # Find which list element is THIS control
        for i in range (0, len(self.control)):
            val = self.control[i]
            if (val == name):
                index = i
                break

        if (index == ""): return("???")    # Error if control not found

        # Now, we should know if it is a VALUE or STATE control
        val = self.states[index]
        if (val != "---"):      # It is a STATE-based control
            val       = self.current[index] # Get current state element by number
            val, null = self.states[index][int(val)]    # Get name of that state
        else: val = self.current[index]
        return val

    def set_value_by_name(self, name, value):
        if    name == "???": return
        elif value == "???": return
        index = ""

        # Find which list element is THIS control
        for i in range (0, len(self.control)):
            val = self.control[i]
            if (val == name):
                index = i
                break

        if (index == ""): return # Error if control not found

        # Now, we should know if it is a VALUE or STATE control
        val = self.states[index]
        if (val != "---"):      # It is a STATE-based control
            pass
            # FIXME FIXME
            #val       = self.current[index] # Get current state element by number
            #val, null = self.states[index][int(val)]    # Get name of that state
        else: self.current[index] = value
        print_info("Saved " + value + " to control " + name)

        # Get the register number for this control
        regnum = self.register[index]
        self.connect("192.168.7.222", 8428)
        self.SET(regnum, value)

    #-----------------------------------------------------------
    # Opens a new connection to a THAT module
    # addr = The IP address of the module
    # port = The network port to connect through
    def connect(self, addr, port):
        addr_from   = ''
        port_from   = 0
        self.addr   = addr
        self.port   = port

        # Create a new TCP socket object
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        try:
            # Setup the source port for the Kernel to use
            self.socket.bind((addr_from, port_from))
        except socket.error, msg:
            tmp = "Module " + self.alias + " socket error:" + str(msg)
            print_error(tmp)

        try:
            # Setup the destination port to connect to
            self.socket.connect((self.addr, self.port))
            tmp = "Module " + self.alias + " connected at " + addr
            print_info(tmp)
        except socket.error, msg:
            self.socket.close()
            tmp = "Module " + self.alias + " connection error:" + str(msg)
            print_error(tmp)

    #-----------------------------------------------------------
    # Disconnects from a THAT module
    def disconnect(self):
        self.socket.close()
        print_info("Module " + self.alias + " Disconnected")

    #-----------------------------------------------------------
    # Sends a @IDD command to a THAT module to get its identification
    def IDD(self):
        data = self.tcp_sendrecv("@IDD")
        n1   = 4
        n2   = data.find("#")

        if (data[0:n1] == "@DAT"):
            tmp = data[n1:n2]

            if (tmp == "$NAME"):
                data         = data[n2:len(data)]
                n1           = data.find("#")
                n2           = data.find("$")
                self.modname = data[n1+1:n2]
                print_info("Module Name: " + self.modname)

                data         = data[n2:len(data)]
                n1           = data.find("$")
                n2           = data.find("#")
                tmp          = data[n1:n2]                

                if (tmp == "$ID"):
                    data       = data[n2:len(data)]
                    n1         = data.find("#")
                    n2         = data.find("$")
                    self.modid = data[n1+1:n2]
                    print_info("Module ID: " + self.modid)

                    data         = data[n2:len(data)]
                    n1           = data.find("$")
                    n2           = data.find("#")
                    tmp          = data[n1:n2]   

                    if (tmp == "$MAN"):
                        data        = data[n2:len(data)]
                        n1          = data.find("#")
                        n2          = len(data)
                        self.modman = data[n1+1:n2]
                        print_info("Module Manufacturer: " + self.modman)

                    else:
                        print_warning("Invalid Manufacturer data")
                else:
                    print_warning("Invalid ID data")
            else:
                print_warning("Invalid NAME data")
        else:
            print_warning("Module provided unexpected response to IDD")

    #-----------------------------------------------------------
    # Sends a @SET command to a THAT module
    # register = Register / Port / memory location to write to
    # setdata  = The actual data or value to set
    def SET(self, register, setdata):
        cmd  = "@SET$" + str(register) + "#" + str(setdata)
        data = self.tcp_sendrecv(cmd)
        n1   = 4
        n2   = data.find("#")

        if (data[0:n1] == "@ACK"):
            tmp = data[n1:n2]

            if (tmp == "$" + str(register)):
                tmp = tmp[1:len(tmp)]
                print_info("Register Name: " + tmp)

                data = data[n2:len(data)]

                if (data == "#" + str(setdata)):
                    data = data[1:len(data)]
                    print_info("Register set to: " + data)
                else:
                    print_error("Invalid response, data was NOT set!")
            else:
                print_warning("Invalid register returned")
        else:
            print_warning("Module ACK not received in response to SET")

    #-----------------------------------------------------------
    # Sends a @GET command to a THAT module
    # register = Register / Port / memory location to read from
    def GET(self, register):
        cmd  = "@GET$" + str(register)
        data = self.tcp_sendrecv(cmd)
        n1   = 4
        n2   = data.find("#")

        if (data[0:n1] == "@DAT"):
            tmp = data[n1:n2]

            if (tmp == "$" + str(register)):
                tmp = tmp[1:len(tmp)]
                print_info("Register Name: " + tmp)

                data = data[n2:len(data)]

                if (data[0] == "#"):
                    data = data[1:len(data)]
                    print_info("Register data is: " + data)
                else:
                    print_error("Invalid response, data was NOT received")
            else:
                print_warning("Invalid register returned")
        else:
            print_warning("Module DAT not received in response to GET")

    #-----------------------------------------------------------
    # Send and receive a set of information to/from a THAT module
    # command = The complete command string to send
    def tcp_sendrecv(self, command):
        tcp_buffer = 1024

        try:
            # Send a message
            self.socket.send(command)
            tmp = "Sent command:" + command
            print_info(tmp)
        except socket.error, msg:
            tmp = "TCP tx error:" + str(msg)
            print_error(tmp)
            return("xxx")

        try:
            # Receive a reply
            indata = self.socket.recv(tcp_buffer)
        except socket.error, msg:
            tmp = "TCP rx error:" + str(msg)
            print_error(tmp)
            return("xxx")

        return(indata)
##########################################################################